#pragma once

#include <map>
#include <mutex>
#include <optional>
#include <string>

#include <RobotAPI/libraries/armem/core/MemoryID.h>

#include "converter/data/Converter.h"
#include "converter/data/object/json/JsonConverter.h"
#include "converter/type/object/json/JsonConverter.h"
#include "extractor/Extractor.h"
#include "filter/Filter.h"

namespace armarx::armem::server::ltm
{
    /// all necessary classes to filter and convert an entry of the ltm to some other format(s)
    class Processors
    {
    public:
        Processors() = default;

        void configure(const nlohmann::json& config);

        std::map<std::string, processor::SnapshotFilter::FilterStatistics> getSnapshotFilterStatistics();

    public:
        // Unique Memory Filters
        std::vector<std::unique_ptr<processor::MemoryFilter>> memFilters;

        // Unique Snapshot filters
        std::vector<std::unique_ptr<processor::SnapshotFilter>> snapFilters;

        // Special Extractors
        std::vector<std::unique_ptr<processor::Extractor>> extractors;

        // Special Converters
        std::vector<std::unique_ptr<processor::DataConverter>> converters;

        // Default converters
        processor::converter::data::object::JsonConverter defaultObjectConverter;
        processor::converter::type::object::JsonConverter defaultTypeConverter;
    };
} // namespace armarx::armem::server::ltm
